@@ -18,34 +18,37 @@ use futures::stream::StreamExt;
 use itertools::Itertools;
 use log::*;
 use maplit::*;
+use pact_plugin_driver::plugin_manager::{load_plugin, shutdown_plugins};
 use regex::Regex;
 use serde_json::Value;
 
 pub use callback_executors::NullRequestFilterExecutor;
 use callback_executors::RequestFilterExecutor;
-use pact_matching::*;
+use pact_matching::{match_response, Mismatch};
 use pact_matching::logging::LOG_ID;
 use pact_models::generators::GeneratorTestMode;
 use pact_models::http_utils::HttpAuth;
 use pact_models::interaction::Interaction;
 use pact_models::json_utils::json_to_string;
 use pact_models::pact::{load_pact_from_url, Pact, read_pact};
+use pact_models::prelude::v4::SynchronousHttp;
 use pact_models::provider_states::*;
-use pact_models::sync_interaction::RequestResponseInteraction;
 use pact_models::v4::interaction::V4Interaction;
 
 use crate::callback_executors::{ProviderStateError, ProviderStateExecutor};
-use crate::messages::{display_message_result, verify_message_from_provider};
+use crate::messages::{display_message_result, verify_message_from_provider, verify_sync_message_from_provider};
 use crate::pact_broker::{Link, PactVerificationContext, publish_verification_results, TestResult};
 pub use crate::pact_broker::{ConsumerVersionSelector, PactsForVerificationRequest};
-use crate::provider_client::{make_provider_request, provider_client_error_to_string};
+use crate::provider_client::make_provider_request;
 use crate::request_response::display_request_response_result;
+use pact_plugin_driver::plugin_models::{PluginDependency, PluginDependencyType};
 
 mod provider_client;
 pub mod pact_broker;
 pub mod callback_executors;
 mod request_response;
 mod messages;
+pub mod selectors;
 
 /// Source for loading pacts
 #[derive(Debug, Clone)]
@@ -72,6 +75,8 @@ pub enum PactSource {
       include_wip_pacts_since: Option<String>,
       /// Provider tags to use in determining pending status for return pacts
       provider_tags: Vec<String>,
+      /// Provider branch to use when publishing verification results
+      provider_branch: Option<String>,
       /// The set of selectors that identifies which pacts to verify
       selectors: Vec<ConsumerVersionSelector>,
       /// HTTP authentication details for accessing the Pact Broker
@@ -90,11 +95,11 @@ impl Display for PactSource {
       PactSource::BrokerUrl(ref provider_name, ref broker_url, _, _) => {
           write!(f, "PactBroker({}, provider_name='{}')", broker_url, provider_name)
       }
-      PactSource::BrokerWithDynamicConfiguration { ref provider_name, ref broker_url,ref enable_pending, ref include_wip_pacts_since, ref provider_tags, ref selectors, ref auth, links: _ } => {
+      PactSource::BrokerWithDynamicConfiguration { ref provider_name, ref broker_url,ref enable_pending, ref include_wip_pacts_since, ref provider_branch, ref provider_tags, ref selectors, ref auth, links: _ } => {
         if let Some(auth) = auth {
-          write!(f, "PactBrokerWithDynamicConfiguration({}, provider_name='{}', enable_ending={}, include_wip_since={:?}, provider_tagcs={:?}, consumer_version_selectors='{:?}, auth={}')", broker_url, provider_name, enable_pending, include_wip_pacts_since, provider_tags, selectors, auth)
+          write!(f, "PactBrokerWithDynamicConfiguration({}, provider_name='{}', enable_ending={}, include_wip_since={:?}, provider_tags={:?}, provider_branch={:?}, consumer_version_selectors='{:?}, auth={}')", broker_url, provider_name, enable_pending, include_wip_pacts_since, provider_tags, provider_branch, selectors, auth)
         } else {
-          write!(f, "PactBrokerWithDynamicConfiguration({}, provider_name='{}', enable_ending={}, include_wip_since={:?}, provider_tagcs={:?}, consumer_version_selectors='{:?}, auth=None')", broker_url, provider_name, enable_pending, include_wip_pacts_since, provider_tags, selectors)
+          write!(f, "PactBrokerWithDynamicConfiguration({}, provider_name='{}', enable_ending={}, include_wip_since={:?}, provider_tags={:?}, provider_branch={:?}, consumer_version_selectors='{:?}, auth=None')", broker_url, provider_name, enable_pending, include_wip_pacts_since, provider_tags, provider_branch, selectors)
 
         }
       }
@@ -121,14 +126,14 @@ pub struct ProviderInfo {
 impl Default for ProviderInfo {
     /// Create a default provider info
   fn default() -> ProviderInfo {
-        ProviderInfo {
-            name: s!("provider"),
-            protocol: s!("http"),
-            host: s!("localhost"),
-            port: Some(8080),
-            path: s!("/")
-        }
+    ProviderInfo {
+      name: "provider".to_string(),
+      protocol: "http".to_string(),
+      host: "localhost".to_string(),
+      port: Some(8080),
+      path: "/".to_string()
     }
+  }
 }
 
 /// Result of performing a match
@@ -202,7 +207,14 @@ impl Clone for MismatchResult {
   fn clone(&self) -> Self {
     match self {
       MismatchResult::Mismatches { mismatches, expected, actual, interaction_id } => {
-        if let Some(ref expected_reqres) = expected.as_request_response() {
+        if expected.is_v4() {
+          MismatchResult::Mismatches {
+            mismatches: mismatches.clone(),
+            expected: expected.boxed(),
+            actual: actual.boxed(),
+            interaction_id: interaction_id.clone()
+          }
+        } else if let Some(ref expected_reqres) = expected.as_request_response() {
           MismatchResult::Mismatches {
             mismatches: mismatches.clone(),
             expected: Box::new(expected_reqres.clone()),
@@ -229,28 +241,30 @@ impl Clone for MismatchResult {
 
 async fn verify_response_from_provider<F: RequestFilterExecutor>(
   provider: &ProviderInfo,
-  interaction: &RequestResponseInteraction,
+  interaction: &SynchronousHttp,
+  pact: &Box<dyn Pact + Send + Sync>,
   options: &VerificationOptions<F>,
   client: &reqwest::Client,
   verification_context: &HashMap<&str, Value>
 ) -> Result<Option<String>, MismatchResult> {
   let expected_response = &interaction.response;
-  match make_provider_request(provider, &pact_matching::generate_request(&interaction.request, &GeneratorTestMode::Provider, &verification_context), options, client).await {
+  let request = pact_matching::generate_request(&interaction.request, &GeneratorTestMode::Provider, &verification_context).await;
+  match make_provider_request(provider, &request, options, client).await {
     Ok(ref actual_response) => {
-      let mismatches = match_response(expected_response.clone(), actual_response.clone());
+      let mismatches = match_response(expected_response.clone(), actual_response.clone(), pact, &interaction.boxed()).await;
       if mismatches.is_empty() {
         Ok(interaction.id.clone())
       } else {
         Err(MismatchResult::Mismatches {
           mismatches,
-          expected: Box::new(interaction.clone()),
-          actual: Box::new(RequestResponseInteraction { response: actual_response.clone(), .. RequestResponseInteraction::default() }),
+          expected: interaction.boxed(),
+          actual: Box::new(SynchronousHttp { response: actual_response.clone(), .. SynchronousHttp::default() }),
           interaction_id: interaction.id.clone()
         })
       }
     },
     Err(err) => {
-      Err(MismatchResult::Error(provider_client_error_to_string(err), interaction.id.clone()))
+      Err(MismatchResult::Error(err.to_string(), interaction.id.clone()))
     }
   }
 }
@@ -266,13 +280,20 @@ async fn execute_state_change<S: ProviderStateExecutor>(
         println!("  Given {}", Style::new().bold().paint(provider_state.name.clone()));
     }
     let result = provider_state_executor.call(interaction_id, provider_state, setup, Some(client)).await;
-    log::debug!("State Change: \"{:?}\" -> {:?}", provider_state, result);
-    result.map_err(|err| MismatchResult::Error(err.description, err.interaction_id))
+    debug!("State Change: \"{:?}\" -> {:?}", provider_state, result);
+    result.map_err(|err| {
+      if let Some(err) = err.downcast_ref::<ProviderStateError>() {
+        MismatchResult::Error(err.description.clone(), err.interaction_id.clone())
+      } else {
+        MismatchResult::Error(err.to_string(), None)
+      }
+    })
 }
 
-async fn verify_interaction<F: RequestFilterExecutor, S: ProviderStateExecutor>(
+async fn verify_interaction<'a, F: RequestFilterExecutor, S: ProviderStateExecutor>(
   provider: &ProviderInfo,
-  interaction: &dyn Interaction,
+  interaction: &(dyn Interaction + Send + Sync),
+  pact: &Box<dyn Pact + Send + Sync + 'a>,
   options: &VerificationOptions<F>,
   provider_state_executor: &Arc<S>
 ) -> Result<Option<String>, MismatchResult> {
@@ -310,16 +331,30 @@ async fn verify_interaction<F: RequestFilterExecutor, S: ProviderStateExecutor>(
   };
 
   info!("Running provider verification for '{}'", interaction.description());
+
   let result = futures::future::ready((provider_states_results.iter()
     .map(|(k, v)| (k.as_str(), v.clone())).collect(), client.clone()))
     .then(|(context, client)| async move {
     let mut result = Err(MismatchResult::Error("No interaction was verified".into(), interaction.id().clone()));
-    if let Some(interaction) = interaction.as_request_response() {
-      result = verify_response_from_provider(provider, &interaction, options, &client, &context).await;
+
+    trace!("Interaction to verify: {:?}", interaction);
+
+    // Verify an HTTP interaction
+    if let Some(interaction) = interaction.as_v4_http() {
+      trace!("Verifying a HTTP interaction");
+      result = verify_response_from_provider(provider, &interaction, &pact.boxed(), options, &client, &context).await;
     }
+    // Verify an asynchronous message (single shot)
     if interaction.is_message() {
-      result = verify_message_from_provider(provider, &interaction.boxed(), options, &client, &context).await;
+      trace!("Verifying an asynchronous message (single shot)");
+      result = verify_message_from_provider(provider, pact, &interaction.boxed(), options, &client, &context).await;
     }
+    // Verify a synchronous message (request/response)
+    if let Some(message) = interaction.as_v4_sync_message() {
+      trace!("a synchronous message (request/response)");
+      result = verify_sync_message_from_provider(provider, pact, message, options, &client, &context).await;
+    }
+
     result
   }).await;
 
@@ -364,9 +399,9 @@ fn display_result(
   println!("      has a matching body ({})", body_result);
 }
 
-fn walkdir(dir: &Path) -> anyhow::Result<Vec<anyhow::Result<Box<dyn Pact>>>> {
+fn walkdir(dir: &Path) -> anyhow::Result<Vec<anyhow::Result<Box<dyn Pact + Send + Sync>>>> {
     let mut pacts = vec![];
-    log::debug!("Scanning {:?}", dir);
+    debug!("Scanning {:?}", dir);
     for entry in fs::read_dir(dir)? {
         let entry = entry?;
         let path = entry.path();
@@ -426,7 +461,7 @@ impl FilterInfo {
         match *self {
             FilterInfo::State(ref s) => s.clone(),
             FilterInfo::DescriptionAndState(_, ref s) => s.clone(),
-            _ => s!("")
+            _ => String::default()
         }
     }
 
@@ -435,7 +470,7 @@ impl FilterInfo {
         match *self {
             FilterInfo::Description(ref s) => s.clone(),
             FilterInfo::DescriptionAndState(ref s, _) => s.clone(),
-            _ => s!("")
+            _ => String::default()
         }
     }
 
@@ -479,7 +514,7 @@ fn filter_interaction(interaction: &dyn Interaction, filter: &FilterInfo) -> boo
   }
 }
 
-fn filter_consumers(consumers: &[String], res: &Result<(Box<dyn Pact>, Option<PactVerificationContext>, PactSource), String>) -> bool {
+fn filter_consumers(consumers: &[String], res: &Result<(Box<dyn Pact + Send + Sync>, Option<PactVerificationContext>, PactSource), String>) -> bool {
   consumers.is_empty() || res.is_err() || consumers.contains(&res.as_ref().unwrap().0.consumer().name)
 }
 
@@ -499,7 +534,9 @@ pub struct VerificationOptions<F> where F: RequestFilterExecutor {
   /// Ignore invalid/self-signed SSL certificates
   pub disable_ssl_verification: bool,
   /// Timeout in ms for verification requests and state callbacks
-  pub request_timeout: u64
+  pub request_timeout: u64,
+  /// Provider branch used when publishing results
+  pub provider_branch: Option<String>,
 }
 
 impl <F: RequestFilterExecutor> Default for VerificationOptions<F> {
@@ -510,6 +547,7 @@ impl <F: RequestFilterExecutor> Default for VerificationOptions<F> {
       build_url: None,
       request_filter: None,
       provider_tags: vec![],
+      provider_branch: None,
       disable_ssl_verification: false,
       request_timeout: 5000
     }
@@ -542,13 +580,13 @@ pub fn verify_provider<F: RequestFilterExecutor, S: ProviderStateExecutor>(
   consumers: Vec<String>,
   options: VerificationOptions<F>,
   provider_state_executor: &Arc<S>
-) -> bool {
+) -> anyhow::Result<bool> {
   match tokio::runtime::Builder::new_multi_thread().enable_all().build() {
     Ok(runtime) => runtime.block_on(
       verify_provider_async(provider_info, source, filter, consumers, options, provider_state_executor)),
     Err(err) => {
       error!("Verify provider process failed to start the tokio runtime: {}", err);
-      false
+      Ok(false)
     }
   }
 }
@@ -561,7 +599,9 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
     consumers: Vec<String>,
     options: VerificationOptions<F>,
     provider_state_executor: &Arc<S>
-) -> bool {
+) -> anyhow::Result<bool> {
+  pact_matching::matchers::configure_core_catalogue();
+
   LOG_ID.scope(format!("verify:{}", provider_info.name), async {
     let pact_results = fetch_pacts(source, consumers).await;
 
@@ -571,7 +611,19 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
     for pact_result in pact_results {
       match pact_result {
         Ok((pact, context, pact_source)) => {
+          if pact.requires_plugins() {
+            info!("Pact file requires plugins, will load those now");
+            for plugin_details in pact.plugin_data() {
+              load_plugin(&PluginDependency {
+                name: plugin_details.name.clone(),
+                version: Some(plugin_details.version.clone()),
+                dependency_type: PluginDependencyType::Plugin
+              }).await?;
+            }
+          }
+
           display_notices(&context, VERIFICATION_NOTICE_BEFORE);
+
           println!("\nVerifying a pact between {} and {}",
           Style::new().bold().paint(pact.consumer().name.clone()),
           Style::new().bold().paint(pact.provider().name.clone()));
@@ -583,15 +635,25 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
               Some(context) => context.verification_properties.pending,
               None => false
             };
-            let result = verify_pact_internal(&provider_info, &filter, pact, &options,
-                                              &provider_state_executor.clone(), pending).await;
-            for result in &result.results {
-              results.push((result.interaction_id.clone(), result.result.clone()));
-              if let Err(error) = &result.result {
-                if result.pending {
-                  pending_errors.push((result.description.clone(), error.clone()));
+            match verify_pact_internal(&provider_info, &filter, pact, &options,
+                                       &provider_state_executor.clone(), pending).await {
+              Ok(result) => for result in &result.results {
+                results.push((result.interaction_id.clone(), result.result.clone()));
+                if let Err(error) = &result.result {
+                  if result.pending {
+                    pending_errors.push((result.description.clone(), error.clone()));
+                  } else {
+                    errors.push((result.description.clone(), error.clone()));
+                  }
+                }
+              }
+              Err(err) => {
+                if pending {
+                  pending_errors.push(("Could not verify the provided pact".to_string(),
+                                       MismatchResult::Error(err.to_string(), None)));
                 } else {
-                  errors.push((result.description.clone(), error.clone()));
+                  errors.push(("Could not verify the provided pact".to_string(),
+                               MismatchResult::Error(err.to_string(), None)));
                 }
               }
             }
@@ -625,15 +687,20 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
       print_errors(&pending_errors);
       println!("\nThere were {} non-fatal pact failures on pending pacts or interactions (see docs.pact.io/pending for more information)\n", pending_errors.len());
     }
-    if !errors.is_empty() {
+
+    let result = if !errors.is_empty() {
       println!("\nFailures:\n");
       print_errors(&errors);
       println!("\nThere were {} pact failures\n", errors.len());
-      false
+      Ok(false)
     } else {
       println!();
-      true
-    }
+      Ok(true)
+    };
+
+    shutdown_plugins();
+
+    result
   }).await
 }
 
@@ -649,7 +716,15 @@ fn print_errors(errors: &Vec<(String, MismatchResult)>) {
             let mismatch = mismatches.next().unwrap();
             println!("    {}.{}) {}", i + 1, j, mismatch.summary());
             println!("           {}", mismatch.ansi_description());
-            for mismatch in mismatches {
+            for mismatch in mismatches.sorted_by(|m1, m2| {
+              match (m1, m2) {
+                (Mismatch::QueryMismatch { parameter: p1, .. }, Mismatch::QueryMismatch { parameter: p2, .. }) => Ord::cmp(&p1, &p2),
+                (Mismatch::HeaderMismatch { key: p1, .. }, Mismatch::HeaderMismatch { key: p2, .. }) => Ord::cmp(&p1, &p2),
+                (Mismatch::BodyMismatch { path: p1, .. }, Mismatch::BodyMismatch { path: p2, .. }) => Ord::cmp(&p1, &p2),
+                (Mismatch::MetadataMismatch { key: p1, .. }, Mismatch::MetadataMismatch { key: p2, .. }) => Ord::cmp(&p1, &p2),
+                _ => Ord::cmp(m1, m2)
+              }
+            }) {
               println!("           {}", mismatch.ansi_description());
             }
 
@@ -664,7 +739,7 @@ fn print_errors(errors: &Vec<(String, MismatchResult)>) {
   }
 }
 
-async fn fetch_pact(source: PactSource) -> Vec<Result<(Box<dyn Pact>, Option<PactVerificationContext>, PactSource), String>> {
+async fn fetch_pact(source: PactSource) -> Vec<Result<(Box<dyn Pact + Send + Sync>, Option<PactVerificationContext>, PactSource), String>> {
   trace!("fetch_pact(source={})", source);
 
   match source {
@@ -695,14 +770,9 @@ async fn fetch_pact(source: PactSource) -> Vec<Result<(Box<dyn Pact>, Option<Pac
           let mut buffer = vec![];
           for result in pacts.iter() {
             match result {
-              Ok((pact, _, links)) => {
-                log::debug!("Got pact with links {:?}", links);
-                if let Ok(pact) = pact.as_request_response_pact() {
-                  buffer.push(Ok((Box::new(pact) as Box<dyn Pact>, None, PactSource::BrokerUrl(provider_name.clone(), broker_url.clone(), auth.clone(), links.clone()))))
-                }
-                if let Ok(pact) = pact.as_message_pact() {
-                  buffer.push(Ok((Box::new(pact) as Box<dyn Pact>, None, PactSource::BrokerUrl(provider_name.clone(), broker_url.clone(), auth.clone(), links.clone()))))
-                }
+              Ok((pact, context, links)) => {
+                trace!("Got pact with links {:?}", pact);
+                buffer.push(Ok((pact.boxed(), context.clone(), PactSource::BrokerUrl(provider_name.clone(), broker_url.clone(), auth.clone(), links.clone()))));
               },
               &Err(ref err) => buffer.push(Err(format!("Failed to load pact from '{}' - {:?}", broker_url, err)))
             }
@@ -712,13 +782,14 @@ async fn fetch_pact(source: PactSource) -> Vec<Result<(Box<dyn Pact>, Option<Pac
         Err(err) => vec![Err(format!("Could not load pacts from the pact broker '{}' - {:?}", broker_url, err))]
       }
     },
-    PactSource::BrokerWithDynamicConfiguration { provider_name, broker_url, enable_pending, include_wip_pacts_since, provider_tags, selectors, auth, links: _ } => {
+    PactSource::BrokerWithDynamicConfiguration { provider_name, broker_url, enable_pending, include_wip_pacts_since, provider_tags, provider_branch, selectors, auth, links: _ } => {
       let result = pact_broker::fetch_pacts_dynamically_from_broker(
         broker_url.as_str(),
         provider_name.clone(),
         enable_pending,
         include_wip_pacts_since,
         provider_tags,
+        provider_branch,
         selectors,
         auth.clone()
       ).await;
@@ -729,14 +800,8 @@ async fn fetch_pact(source: PactSource) -> Vec<Result<(Box<dyn Pact>, Option<Pac
           for result in pacts.iter() {
             match result {
               Ok((pact, context, links)) => {
-                log::debug!("Got pact with links {:?}", links);
-                if let Ok(pact) = pact.as_request_response_pact() {
-
-                  buffer.push(Ok((Box::new(pact) as Box<dyn Pact>, context.clone(), PactSource::BrokerUrl(provider_name.clone(), broker_url.clone(), auth.clone(), links.clone()))))
-                }
-                if let Ok(pact) = pact.as_message_pact() {
-                  buffer.push(Ok((Box::new(pact) as Box<dyn Pact>, context.clone(), PactSource::BrokerUrl(provider_name.clone(), broker_url.clone(), auth.clone(), links.clone()))))
-                }
+                trace!("Got pact with links {:?}", pact);
+                buffer.push(Ok((pact.boxed(), context.clone(), PactSource::BrokerUrl(provider_name.clone(), broker_url.clone(), auth.clone(), links.clone()))));
               },
               &Err(ref err) => buffer.push(Err(format!("Failed to load pact from '{}' - {:?}", broker_url, err)))
             }
@@ -751,8 +816,8 @@ async fn fetch_pact(source: PactSource) -> Vec<Result<(Box<dyn Pact>, Option<Pac
 }
 
 async fn fetch_pacts(source: Vec<PactSource>, consumers: Vec<String>)
-  -> Vec<Result<(Box<dyn Pact>, Option<PactVerificationContext>, PactSource), String>> {
-  trace!("fetch_pacts(source={:?}, consumers={:?})", source, consumers);
+  -> Vec<Result<(Box<dyn Pact + Send + Sync>, Option<PactVerificationContext>, PactSource), String>> {
+  trace!("fetch_pacts(source={}, consumers={:?})", source.iter().map(|s| s.to_string()).join(", "), consumers);
 
   futures::stream::iter(source)
     .then(|pact_source| async {
@@ -786,19 +851,18 @@ pub struct VerificationResult {
 pub async fn verify_pact_internal<'a, F: RequestFilterExecutor, S: ProviderStateExecutor>(
   provider_info: &ProviderInfo,
   filter: &FilterInfo,
-  pact: Box<dyn Pact + 'a>,
+  pact: Box<dyn Pact + Send + Sync + 'a>,
   options: &VerificationOptions<F>,
   provider_state_executor: &Arc<S>,
   pending: bool
-) -> VerificationResult {
-  let results: Vec<(&dyn Interaction, Result<Option<String>, MismatchResult>)> = futures::stream::iter(
-    pact.interactions().iter().cloned()
-  )
-    .filter(|interaction| futures::future::ready(filter_interaction(*interaction, filter)))
-    .then( |interaction| async move {
-      verify_interaction(provider_info, interaction, options, provider_state_executor)
-        .then(|result| futures::future::ready((interaction, result)))
-        .await
+) -> anyhow::Result<VerificationResult> {
+  let interactions = pact.interactions();
+
+  let results: Vec<(Box<dyn Interaction + Send + Sync>, Result<Option<String>, MismatchResult>)> =
+    futures::stream::iter(interactions.iter().map(|i| (&pact, i)))
+    .filter(|(_, interaction)| futures::future::ready(filter_interaction(interaction.as_ref(), filter)))
+    .then( |(pact, interaction)| async move {
+      (interaction.boxed(), verify_interaction(provider_info, interaction.as_ref(), &pact.boxed(), options, provider_state_executor).await)
     })
     .collect()
     .await;
@@ -815,6 +879,8 @@ pub async fn verify_pact_internal<'a, F: RequestFilterExecutor, S: ProviderState
     }
     description.push_str(" - ");
     description.push_str(&interaction.description());
+
+    println!();
     if interaction.pending() {
       println!("  {} {}", interaction.description(), Yellow.paint("[PENDING]"));
     } else {
@@ -856,25 +922,35 @@ pub async fn verify_pact_internal<'a, F: RequestFilterExecutor, S: ProviderState
 
   println!();
 
-  VerificationResult { results: errors }
+  Ok(VerificationResult { results: errors })
 }
 
 fn display_comments(interaction: Box<dyn V4Interaction>) {
   let comments = interaction.comments();
   if !comments.is_empty() {
     if let Some(testname) = comments.get("testname") {
-      println!("\n  Test Name: {}", json_to_string(testname));
+      let s = json_to_string(testname);
+      if !s.is_empty() {
+        println!("\n  Test Name: {}", s);
+      }
     }
     if let Some(comment_text) = comments.get("text") {
-      println!("\n  Comments:");
       match comment_text {
-        Value::Array(comment_text) => for value in comment_text {
-          println!("    {}", json_to_string(value));
+        Value::Array(comment_text) => if !comment_text.is_empty() {
+          println!("\n  Comments:");
+          for value in comment_text {
+            println!("    {}", json_to_string(value));
+          }
+          println!();
         }
-        _ => println!("    {}", comment_text)
+        Value::String(comment) => if !comment.is_empty() {
+          println!("\n  Comments:");
+          println!("    {}", comment);
+          println!();
+        }
+        _ => {}
       }
     }
-    println!();
   }
 }
 
@@ -904,7 +980,8 @@ async fn publish_result<F: RequestFilterExecutor>(
       result,
       provider_version,
       options.build_url.clone(),
-      options.provider_tags.clone()
+      options.provider_tags.clone(),
+      options.provider_branch.clone()
     ).await;
 
     match &publish_result {
